package CBTagFS;

use 5.010000;
use strict;
use warnings;

our $VERSION = '0.2';

use CBTags;
use Fuse;
use Fcntl;
use Errno;

sub new
{
	my $class = shift;
	my $dbi = shift;

	my $self = {};
	bless $self;

	$self->{'cbtags'} = CBTags->new($dbi);

	return $self;
}

sub get_dirs
{
	my ($self, @tags) = @_;

	my $cbtags = $self->{'cbtags'};

	return $cbtags->all_tags if $#tags < 0;

	return $cbtags->get_remaining_valid_tags(@tags);
}

sub f_getdir
{
	my ($self, $dir) = @_;

	my $cbtags = $self->{'cbtags'};

	my @tags = split('/', $dir);
	shift @tags;

	my @dirs = $self->get_dirs(@tags);
	my @files = $cbtags->get_files(@tags);

	map { s|.*/|| } @files;

	return @dirs, @files, 0;
}

sub is_dir
{
	my ($self, $path) = @_;

	my $cbtags = $self->{'cbtags'};

	my @tags = split '/', $path;

	foreach my $tag (@tags)
	{
		next unless $tag;
		return 0 unless $cbtags->tag_exists($tag);
	}

	return 1;
}

sub dissect
{
	my $path = shift;

	$path =~ m|(.*)/([^/]+)$|;

	my $dir = $1;
	my $name = $2;

	return ($dir, $name);
}

sub is_file
{
	my ($self, $path) = @_;

	my $cbtags = $self->{'cbtags'};

	my ($dir, $name) = dissect($path);

	return 0 unless $self->is_dir($dir);
	return $cbtags->file_location($name);
}

sub statfs
{
	return (255, 1, 1, 1, 1, 2);
}

sub basename
{
	my $file = shift;

	$file =~ s|.*/||;

	return $file;
}

sub file_exists
{
	my ($self, $path) = @_;

	my $cbtags = $self->{'cbtags'};

	my ($dir, $name) = dissect($path);

	return 0 unless $cbtags->file_location($name);

	my @tags = split '/', $dir;

	foreach my $tag (@tags)
	{
		next unless $tag;
		return 0 unless $cbtags->has_tag($name, $tag);
	}

	return 1;
}

sub f_getattr
{
	my ($self, $file) = @_;

	my $cbtags = $self->{'cbtags'};

	my $basename = basename($file);

	my $location = $cbtags->file_location($basename);

	my @tags = split '/', $file;

	pop @tags if $location;

	foreach my $tag (@tags)
	{
		next unless $tag;
		return -Errno::ENOENT() unless $cbtags->tag_exists($tag);
		if ($location)
		{
			return -Errno::ENOENT() unless $cbtags->has_tag($basename, $tag);
		}
	}

	my $mode = $location ? Fcntl::S_IFLNK | 0777 : Fcntl::S_IFDIR | 0755;

	return (0, 0, $mode, 1, 0, 0, 0, 10, time(), time(), time(), 1024, 1);
}

sub f_readlink
{
	my ($self, $file) = @_;

	my $cbtags = $self->{'cbtags'};

	return $cbtags->file_location(basename($file));
}

sub f_unlink
{
	my ($self, $file) = @_;

	my $cbtags = $self->{'cbtags'};

	my ($dir, $name) = dissect($file);

	unless ($dir)
	{
		$cbtags->delete_file($name);
		return 0;
	}

	my $tag = basename($dir);

	$cbtags->remove_tag($name, $tag);
	return 0;
}

sub f_rename
{
	my ($self, $old, $new) = @_;

	my $cbtags = $self->{'cbtags'};

	my $basename = basename($old);

	$new =~ s|/[^/]*$||;

	my @tags = split '/', $new;

	foreach my $tag (@tags)
	{
		next unless $tag;
		$cbtags->add_tag($basename, $tag);
	}

	return 0;
}

sub f_symlink
{
	my ($self, $location, $link) = @_;

	my $cbtags = $self->{'cbtags'};

	my ($dir, $name) = dissect($link);

	my $file_id = $cbtags->create_file($name, $location);

	my @tags = split '/', $dir;
	foreach my $tag (@tags)
	{
		next unless $tag;
		$cbtags->add_tag($name, $tag);
	}

	return 0;
}

sub f_mkdir
{
	my ($self, $path, $mode) = @_;

	my $cbtags = $self->{'cbtags'};

	my $basename = basename($path);

	$cbtags->create_tag($basename);

	return 0;
}

sub f_rmdir
{
	my ($self, $path) = @_;

	my $cbtags = $self->{'cbtags'};

	my $basename = basename($path);

	$cbtags->delete_tag($basename);

	return 0;
}

sub f_chmod
{
	my ($self, $path, $mode) = @_;

	# swallow up chmods
	return 0;
}

sub f_chown
{
	my ($self, $path, $uid, $gid) = @_;

	# swallow up chowns
	return 0;
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CBTagFS - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CBTagsFS;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CBTagFS, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Jacob Beacham, E<lt>jakob@codebreath.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009 by Jacob Beacham

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.


=cut
