package CBTags;

use 5.010000;
use strict;
use warnings;

our $VERSION = '0.2';

use DBI;

sub new
{
	my $class = shift;
	my $dbi = shift;

	my $self = {};
	bless $self;

	my $dbh = DBI->connect($dbi) or die "Couldn't connect to database: " . DBI->errstr;
	$self->{'db'} = $dbh;

	$self->init_schema;

	return $self;
}

sub init_schema
{
	my ($self, @args) = @_;

	$self->_do(
		'create table if not exists file(
			name NOT NULL UNIQUE,
			path NOT NULL
		)');

	$self->_do(
		'create table if not exists tag(
			name NOT NULL UNIQUE
		)');

	$self->_do(
		'create table if not exists links(
			file INTEGER NOT NULL REFERENCES file(rowid),
			tag INTEGER NOT NULL REFERENCES tag(rowid)
		)');
}

sub create_tag
{
	my ($self, @args) = @_;

	my $tag = shift @args;

	$self->_do('INSERT OR IGNORE INTO tag VALUES (?)', $tag);
}

sub delete_tag
{
	my ($self, @args) = @_;

	my $tag = shift @args;

	my $tag_id = $self->_get_tag_id($tag);

	$self->_do('DELETE FROM links WHERE tag = ?', $tag_id);
	$self->_do('DELETE FROM tag WHERE name = ?', $tag);
}

sub add_tag
{
	my ($self, @args) = @_;

	my $file = shift @args;
	my $tag = shift @args;

	my $file_id = $self->_get_file_id($file);
	my $tag_id = $self->_get_tag_id($tag);

	# don't create duplicates
	$self->_fetchone('SELECT rowid FROM links WHERE file = ? AND tag = ?', $file_id, $tag_id) && return;

	$self->_do('INSERT OR IGNORE INTO links (tag, file) VALUES (?, ?)', $tag_id, $file_id);
}

sub remove_tag
{
	my ($self, @args) = @_;

	my $file = shift @args;
	my $tag = shift @args;

	my $file_id = $self->_get_file_id($file);
	my $tag_id = $self->_get_tag_id($tag);

	$self->_do('DELETE FROM links WHERE file = ? AND tag = ?', $file_id, $tag_id);
}

sub get_tags
{
	my ($self, @args) = @_;

	my $file = shift @args;

	return $self->_fetchcols('SELECT tag.name FROM file JOIN links ON (file.rowid = links.file) JOIN tag ON (links.tag = tag.rowid) WHERE file.name = ?', $file);
}

sub get_files
{
	my ($self, @args) = @_;

	my @tags = @args;

	return $self->_fetchcols('SELECT name FROM file') if ($#tags < 0);

	my @intersects;
	foreach (@tags)
	{
		push @intersects, 'SELECT file FROM links WHERE tag = ?';
	}

	my $sql = 'SELECT file.name FROM (' . join(' INTERSECT ', @intersects) . ') AS links JOIN file ON (links.file = file.rowid)';

	return $self->_fetchcols($sql, map { $self->_get_tag_id($_) } @tags);
}

sub get_remaining_valid_tags
{
	my ($self, @args) = @_;

	my @tags = @args;

	my @intersects;
	my @not_in;
	my @tag_ids;
	foreach (@tags)
	{
		push @intersects, 'SELECT file FROM links WHERE tag = ?';
		push @not_in, '?';
		push @tag_ids, $self->_get_tag_id($_);
	}

	my $sql = 'SELECT DISTINCT name FROM (' . join(' INTERSECT ', @intersects) . ') AS f JOIN (SELECT * FROM links JOIN tag ON (links.tag = tag.rowid) WHERE tag NOT IN (' . join(',', @not_in) . ')) AS t ON (f.file = t.file)';

	return $self->_fetchcols($sql, @tag_ids, @tag_ids);
}

sub all_tags
{
	my ($self, @args) = @_;

	return $self->_fetchcols('SELECT name FROM tag');
}

sub tag_exists
{
	return _get_tag_id(@_);
}

sub file_location
{
	my ($self, @args) = @_;

	my $file = shift @args;

	return $self->_fetchone('SELECT path FROM file WHERE name = ?', $file);
}

sub create_file
{
	my ($self, @args) = @_;

	my $name = shift @args;
	my $path = shift @args;

	$self->_do('INSERT OR IGNORE INTO file (name, path) VALUES (?, ?)', $name, $path);

	return $self->_get_file_id($name);
}

sub delete_file
{
	my ($self, @args) = @_;

	my $file = shift @args;

	my $file_id = $self->_get_file_id($file);
	$self->_do('DELETE FROM file WHERE name = ?', $file);
	$self->_do('DELETE FROM links WHERE file = ?', $file_id);
}

sub has_tag
{
	my ($self, @args) = @_;

	my $file = shift @args;
	my $tag = shift @args;

	return $self->_fetchone('SELECT links.rowid FROM file JOIN links ON (file.rowid = links.file) JOIN tag ON (links.tag = tag.rowid) WHERE file.name = ? AND tag.name = ?', $file, $tag);
}

# helpers

sub _get_tag_id
{
	my ($self, @args) = @_;

	my $tag = shift @args;

	return $self->_fetchone('SELECT rowid FROM tag WHERE name = ?', $tag);
}

sub _get_file_id
{
	my ($self, @args) = @_;

	my $file = shift @args;

	return $self->_fetchone('SELECT rowid FROM file WHERE name = ?', $file);
}

# DB helpers

sub _fetchone
{
	my ($self, @args) = @_;

	my $sql = shift @args;

	my $sth = $self->{'db'}->prepare($sql);
	$sth->execute(@args);

	my @row = $sth->fetchrow_array;
	return $row[0];
}

sub _fetchcols
{
	my ($self, @args) = @_;

	my $sql = shift @args;

	my $sth = $self->{'db'}->prepare($sql);
	$sth->execute(@args);

	my @cols;
	while ((my $col) = $sth->fetchrow_array)
	{
		push @cols, $col;
	}
	return @cols;
}

sub _do
{
	my ($self, @args) = @_;

	my $sql = shift @args;

	my $sth = $self->{'db'}->prepare($sql) or return undef;
	$sth->execute(@args) or return undef;

	return $sth->rows;
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

CBTags - Perl extension for blah blah blah

=head1 SYNOPSIS

  use CBTags;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for CBTags, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Jacob Beacham, E<lt>jakob@codebreath.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009 by Jacob Beacham

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.


=cut
